/* coreography, a tool for examining ELF based memory images.
 * Copyright (C) 2003  John Ventura ventura@engination.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <elf.h>
#include <sys/procfs.h>
#include <sys/types.h>

void *getauxinfo(uint8_t * buf, int len, int type);

/* This function dumps the first 80 characters of the command like that
 * was used to launch the command that generated the core file.
 * This info is stored in the core file headers.  It is also in the
 * "stack" of the core dump.
 */

int argdump(uint8_t * buf, int len, int binary, int usepid, int pid,
	    FILE * desc)
{
    int i;
    int aux_offset = 0;
    int auxsize = 0;
    char procfilename[512];
    char c;
    struct elf_prpsinfo *prpsinfo = NULL;
    Elf32_Ehdr *elfheader;
    Elf32_Phdr *programheader;
    FILE *source;

    if (usepid) {
	snprintf(procfilename, sizeof(procfilename), "/proc/%i/cmdline",
		 pid);
	source = fopen(procfilename, "rb");
	if (source == NULL) {
	    fprintf(stderr, "can't get process info\n");
	    exit(1);
	}
	while (fread(&c, 1, 1, source)) {
	    if (c == 0)
		c = 0x20;	/* Lets hope IFS doesn't is space */
	    putc(c, desc);
	}
	fprintf(desc, "\n");
	fclose(source);
	return (0);
    } else {
	elfheader = (Elf32_Ehdr *) buf;
	programheader = (Elf32_Phdr *) (buf + elfheader->e_phoff);

	if (elfheader->e_type != ET_CORE) {
	    fprintf(stderr, "this is not a core dump\n");
	    exit(1);
	}
	/* get location and size of auxiliary data */
	for (i = 0; i < elfheader->e_phnum; i++) {
	    if (programheader[i].p_type == PT_NOTE) {
		aux_offset = programheader[i].p_offset;
		auxsize = programheader[i].p_filesz;
	    }
	}
	prpsinfo =
	    (struct elf_prpsinfo *) getauxinfo((buf + aux_offset), auxsize,
					       NT_PRPSINFO);
	fprintf(desc, "%s\n", prpsinfo->pr_psargs);
    }
    return (0);
}
