/* coreography, a tool for examining ELF based memory images.
 * Copyright (C) 2003  John Ventura ventura@engination.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <elf.h>
#include <sys/types.h>
#include "addresses.h"

/* This function takes a buffer which *should* contain a core core dump.
 * It then parses the buffer and returns an offset to the internal headers.
 * "Internal" headers should be the same as the "elf and program" headers
 * of the original executabble.
 */

int getinternaloffset(uint8_t * buf)
{
    int i;
    Elf32_Ehdr *elfheader;
    Elf32_Phdr *programheader;

    elfheader = (Elf32_Ehdr *) buf;
    programheader = (Elf32_Phdr *) (buf + elfheader->e_phoff);
    if (elfheader->e_type != ET_CORE)
	return (0);
    for (i = 0; i < elfheader->e_phnum; i++) {
	if ((programheader[i].p_vaddr == START_TEXT_LINUX)
	    && (programheader[i].p_flags & PF_X)) {
	    return (programheader[i].p_offset);
	}
    }
    return (0);
}
