
/* coreography, a tool for examining ELF based memory images.
 * Copyright (C) 2003  John Ventura ventura@engination.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sys/types.h>
#include <sys/ptrace.h>
#include <sys/wait.h>
#include <sys/user.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <elf.h>
#include <fcntl.h>
#include "addresses.h"

/* this function returns a pointer to a copy of a segment of memory
 * from a running process (specified by pid).  The memory segment of
 * pid is defined by the start address addr and the length len.
 */

uint8_t *getprocseg(int pid, uint32_t addr, int len)
{
    int wait_val;
    int i;
    uint32_t foo = 0;
    uint8_t *buf;

    buf = (uint8_t *) malloc(len);
    if (buf == NULL) {
	exit(1);
    }
    memset(buf, (len), 0x00);

    /* Attach to the process and wait to proceed */
    if (ptrace(PTRACE_ATTACH, pid, NULL, NULL) < 0) {
	fprintf(stderr, "can't attach to process %i\n", pid);
	exit(1);
    }
    wait(&wait_val);

    i = 0;
    for (i = 0; i <= len; i += 4) {
	foo = ptrace(PTRACE_PEEKDATA, pid, (addr + i), NULL);

	memcpy((buf + i), &foo, 4);
    }

    ptrace(PTRACE_DETACH, pid, NULL, NULL);
    return (buf);
}
