/* coreography, a tool for examining ELF based memory images.
 * Copyright (C) 2003  John Ventura ventura@engination.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <elf.h>
#include <sys/types.h>
#include "cmd.h"
#include "addresses.h"

int segdumpascii(uint8_t * buf, int len, uint32_t address, FILE * desc);
int segdumpbinary(uint8_t * buf, int len, FILE * desc);

/* How many columns do you want displayed when "printable" ascii output
 * is happening?  16 works well.
 */
#define COLUMNS	0x10

/* ctype.h has a lot of macros like this, but this is how I wanted it.
 * if it's a letter, number or punctuation, return true.
 */
#define ischar(c)((c > 0x20) && (c < 0x7e))

/* This function displays the contents of a given area of memory (buf),
 * that is len bytes in length in a visual format that some humans
 * might like.  The first column is the memory address of what is
 * being displayed.  The initial valuee of thiss is specified in
 * address.  The next several columns, the number of which is defined
 * by COLUMNS, are the 8 bit represntation of each sequential byte.
 * The final column is a the ascii representation of what this data
 * might look like if it were typed.
 */

int segdumpascii(uint8_t * buf, int len, uint32_t address, FILE * desc)
{
    int i;
    int x;
    int pos;
    for (i = pos = 0; i < len; i++) {
	if (pos == 0)
	    fprintf(desc, "%08x\x20\x20\x20", address);
	fprintf(desc, "%02x ", buf[i]);
	address++;
	pos++;
	if (pos == COLUMNS) {
	    fprintf(desc, "\t");
	    for (x = 1; x <= pos; x++) {
		if (ischar(buf[i - (pos - x)]))
		    fprintf(desc, "%c", (buf[i - (pos - x)]));
		else
		    fprintf(desc, ".");
	    }
	    fprintf(desc, "\n");
	    pos = 0;
	}
    }
    if (pos != COLUMNS) {
	for (i = 0; i < (COLUMNS - pos); i++) {
	    fprintf(desc, "   ");
	}
	fprintf(desc, "\t");
	for (x = 1; x <= pos; x++) {
	    if (ischar(buf[i - (pos - x)]))
		fprintf(desc, "%c", (buf[i - (pos - x)]));
	    else
		fprintf(desc, ".");
	}
	fprintf(desc, "\n");
    }
    return (1);
}

/* This just dumps len bytes the contents of a memory location (buf)
 * into a file descriptor.
 */
int segdumpbinary(uint8_t * buf, int len, FILE * desc)
{
    int i;
    for (i = 0; i < len; i++)
	fprintf(desc, "%c", buf[i]);
    return (1);
}
