/* coreography, a tool for examining ELF based memory images.
 * Copyright (C) 2003  John Ventura ventura@engination.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <elf.h>
#include <ctype.h>
#include <stdint.h>
#include <sys/types.h>
#include "cmd.h"
#include "addresses.h"

/*    This function allows the user to run commands on a specified
 *    chunk of memory.
 */

int strings(uint8_t * buf, int len, uint32_t address, FILE * desc);
int segdumpascii(uint8_t * buf, int len, uint32_t address, FILE * desc);
int segdumpbinary(uint8_t * buf, int len, FILE * desc);
int search(uint8_t * buf, int buflen, uint32_t address, uint8_t * query,
	   int querylen, FILE * desc);
uint8_t *getprocseg(int pid, uint32_t addr, int len);

int selectbyaddr(uint8_t * buf, int buflen, uint32_t address, uint32_t len,
		 int binary, int cmdval, uint8_t * query, int querylen,
		 int usepid, int pid, FILE * desc)
{
    int rval = 0;
    int i;
    uint8_t *startbuf;
    uint32_t endsegment;
    uint32_t vaddr;
    int memsize = 0;
    Elf32_Ehdr *elfheader;
    Elf32_Phdr *programheader;

    if (usepid) {
	startbuf = getprocseg(pid, address, len);
	vaddr = address;
	memsize = len;
	switch (cmdval) {
	case CMD_STRINGS:
	    rval += strings(startbuf, memsize, vaddr, desc);
	    break;
	case CMD_SEGDUMP:
	    if (binary)
		rval += segdumpbinary(startbuf, memsize, desc);
	    else
		rval += segdumpascii(startbuf, memsize, vaddr, desc);
	    break;
	case CMD_SEARCH:
	    rval +=
		search(startbuf, memsize, vaddr, query, querylen, desc);
	    break;
	}
    } else {
	elfheader = (Elf32_Ehdr *) buf;
	programheader = (Elf32_Phdr *) (buf + elfheader->e_phoff);

	for (i = 0; i < elfheader->e_phnum; i++) {
	    endsegment =
		programheader[i].p_vaddr + programheader[i].p_memsz;

	    if ((address >= programheader[i].p_vaddr) &&
		(address <= endsegment)) {
		startbuf =
		    (buf + programheader[i].p_offset +
		     (address - programheader[i].p_vaddr));
		/* If the user tries to look at memory that is beyond
		 * the bounds of the segment, don't let it happen.
		 */
		if ((programheader[i].p_vaddr + programheader[i].p_memsz)
		    >= (address + len))
		    memsize = len;
		else
		    memsize =
			((programheader[i].p_vaddr +
			  programheader[i].p_memsz) - address);
		vaddr = address;
		switch (cmdval) {
		case CMD_STRINGS:
		    rval += strings(startbuf, memsize, vaddr, desc);
		    break;
		case CMD_SEGDUMP:
		    if (binary)
			rval += segdumpbinary(startbuf, memsize, desc);
		    else
			rval +=
			    segdumpascii(startbuf, memsize, vaddr, desc);
		    break;
		case CMD_SEARCH:
		    rval +=
			search(startbuf, memsize, vaddr, query, querylen,
			       desc);
		    break;
		}

	    }
	}
    }

    return (rval);
}
