/* coreography, a tool for examining ELF based memory images.
 * Copyright (C) 2003  John Ventura ventura@engination.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <elf.h>
#include <ctype.h>
#include <sys/types.h>
#include "cmd.h"
#include "addresses.h"

int strings(uint8_t * buf, int len, uint32_t address, FILE * desc);

#define ischar(c)((c > 0x20) && (c < 0x7e))
/* yes, there are macros like ischar() in ctype.h, but I don't like them */

/* This function is like strings(1), except in stead of using a file as
 * input, it looks at len bytes of a memory location starting at *buf.
 * The output is displayed along with a 32bit representation of the pointer
 * to that string in memory.   This  is calculated with the base address
 * specified in the third argument.
 */

int strings(uint8_t * buf, int len, uint32_t address, FILE * desc)
{
    int rval = 0;
    int i;
    int stroff = 0;
    char *stringptr = NULL;

    for (i = 0; i < len; i++) {
	if ((buf[i] == 0) || (buf[i] == 0x0a)) {
	    if (stringptr != NULL) {
		rval++;
		fprintf(desc, "%08x\t", (stroff + address));
		while (*stringptr) {
		    if (ischar(*stringptr))
			fprintf(desc, "%c", *stringptr);
		    else
			fprintf(desc, "\\x%02x", *stringptr);
		    stringptr++;
		}
		fprintf(desc, "\n");
	    }
	    stringptr = NULL;
	}
	if (!isprint(buf[i])) {
	    stringptr = NULL;
	}
	if ((stringptr == NULL) && (isprint(buf[i]))) {
	    stringptr = &buf[i];
	    stroff = i;
	}
    }

    return (rval);
}
